Package: logging
Version: 0.10-111
Title: R Logging Package
Authors@R: c(
    person("Mario", "Frasca", email = "mariotomo@gmail.com", role = c("aut", "cre")),
    person("Walerian", "Sokolowski", email = "r-logging@wlogsolutions.com", role = c("aut"))
    )
Description: Pure R implementation of the ubiquitous log4j package. It offers hierarchic 
  loggers, multiple handlers per logger, level based filtering, space handling in messages 
  and custom formatting.
URL: https://github.com/WLOGSolutions/r-logging
BugReports: https://github.com/WLOGSolutions/r-logging/issues
License: GPL-3
Encoding: UTF-8
Language: en-US
Depends: R (>= 3.2.0)
Imports: methods
Suggests: testthat, crayon
RoxygenNote: 6.1.1
NeedsCompilation: no
Packaged: 2026-05-21 14:33:31 UTC; mario
Author: Mario Frasca [aut, cre],
  Walerian Sokolowski [aut]
Maintainer: Mario Frasca <mariotomo@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-21 21:50:30 UTC
Built: R 4.5.2; ; 2026-05-22 05:38:27 UTC; unix
