# Generated by roxygen2: do not edit by hand

S3method(updateOptions,Logger)
S3method(updateOptions,character)
S3method(updateOptions,environment)
export(addHandler)
export(basicConfig)
export(getHandler)
export(getLogger)
export(levellog)
export(logReset)
export(logdebug)
export(logerror)
export(logfine)
export(logfiner)
export(logfinest)
export(loginfo)
export(loglevels)
export(logwarn)
export(removeHandler)
export(resetMsgComposer)
export(setLevel)
export(setMsgComposer)
export(updateOptions)
export(writeToConsole)
export(writeToFile)
import(methods)
