% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{logging-entrypoints}
\alias{logging-entrypoints}
\alias{logdebug}
\alias{logfinest}
\alias{logfiner}
\alias{logfine}
\alias{loginfo}
\alias{logwarn}
\alias{logerror}
\alias{levellog}
\title{Entry points for logging actions}
\usage{
logdebug(msg, ..., logger = "")

logfinest(msg, ..., logger = "")

logfiner(msg, ..., logger = "")

logfine(msg, ..., logger = "")

loginfo(msg, ..., logger = "")

logwarn(msg, ..., logger = "")

logerror(msg, ..., logger = "")

levellog(level, msg, ..., logger = "")
}
\arguments{
\item{msg}{the textual message to be output, or the format for the \dots
arguments}

\item{...}{if present, msg is interpreted as a format and the \dots values
are passed to it to form the actual message.}

\item{logger}{the name of the logger to which we pass the record}

\item{level}{The logging level}
}
\description{
Generate a log record and pass it to the logging system.\cr
}
\details{
A log record gets timestamped and will be independently formatted by each
of the handlers handling it.\cr

Leading and trailing whitespace is stripped from the final message.
}
\examples{
logReset()
addHandler(writeToConsole)
loginfo('this goes to console')
logdebug('this stays silent')

}
